==> cx88-1.0.2.tar.gz: cx88-1.0.2/client/cx88_file_capture.cpp <==
/*-
 * Copyright 2007 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>
#include <fcntl.h>
#include "cx88_file_capture.h"

cx88_file_capture::cx88_file_capture(const char *filename, int &error, size_t cache)
   : cx88_capture(error),
     m_file(-1),
     m_cache(NULL),
     m_cachesize(cache),
     m_usedcache(0)
{
   if (error)
   {
      return;
   }
   if ((cache > 0) && ((m_cache = malloc(cache)) == NULL))
   {
      CX88ERR << "Unable to allocate memory for file cache" << endl;
      error = ENOMEM;
      return;
   }
   int flags = O_APPEND;
   if (strncmp(filename, "/dev/", 5) != 0)
   {
      flags |= (O_CREAT | O_TRUNC | O_RDWR);
   }
   if ((m_file = open(filename, flags, 0644)) < 0)
   {
      CX88ERR << "Unable open output file " << filename << " for capture: " << strerror(errno) << endl;
      error = ENOENT;
      return;
   }
}

cx88_file_capture::~cx88_file_capture(void)
{
   stop();
   if (m_cache != NULL)
   {
      free(m_cache);
      m_cache = NULL;
   }
   if (m_file >= 0)
   {
      close(m_file);
      m_file = -1;
   }
}

void cx88_file_capture::flush(void)
{
   if (m_usedcache > 0)
   {
      write(m_file, m_cache, m_usedcache);
      m_usedcache = 0;
   }
}

void cx88_file_capture::process_buffer(void *data, size_t num_bytes, size_t packet_size)
{
   if ((m_cachesize - m_usedcache) < num_bytes)
   {
      flush();
   }
   if (num_bytes >= m_cachesize)
   {
      write(m_file, data, num_bytes);
      return;
   }
   memcpy(((char *)m_cache) + m_usedcache, data, num_bytes);
   m_usedcache += num_bytes;
}

==> cx88-1.1.0.tar.gz: cx88-1.1.0/client/cx88_file_capture.cpp <==
/*-
 * Copyright 2007 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include "cx88_file_capture.h"

cx88_file_capture::cx88_file_capture(const char *filename, int &error, size_t cache, int flags)
   : cx88_capture(error),
     m_file(-1),
     m_cache(NULL),
     m_cachesize(cache),
     m_usedcache(0)
{
   if (error)
   {
      return;
   }
   if ((cache > 0) && ((m_cache = malloc(cache)) == NULL))
   {
      CX88ERR << "Unable to allocate memory for file cache" << endl;
      error = ENOMEM;
      return;
   }
   if (flags == 0)
   {
      flags = O_APPEND;
      if (strncmp(filename, "/dev/", 5) != 0)
      {
         flags |= (O_CREAT | O_WRONLY);
      }
   }
   if ((m_file = open(filename, flags, 0644)) < 0)
   {
      CX88ERR << "Unable open output file " << filename << " for capture: " << strerror(errno) << endl;
      error = ENOENT;
      return;
   }
   signal(SIGPIPE, SIG_IGN);
}

cx88_file_capture::~cx88_file_capture(void)
{
   stop();
   if (m_cache != NULL)
   {
      free(m_cache);
      m_cache = NULL;
   }
   if (m_file >= 0)
   {
      close(m_file);
      m_file = -1;
   }
   signal(SIGPIPE, SIG_DFL);
}

void cx88_file_capture::flush(void)
{
   if (m_usedcache > 0)
   {
      write(m_file, m_cache, m_usedcache);
      m_usedcache = 0;
   }
}

void cx88_file_capture::process_buffer(void *data, size_t num_bytes, size_t packet_size)
{
   if ((m_cachesize - m_usedcache) < num_bytes)
   {
      flush();
   }
   if (num_bytes >= m_cachesize)
   {
      ssize_t numbytes = write(m_file, data, num_bytes);
      if (numbytes < 0)
      {
         DIAGNOSTIC(CX88ERR << "Warning: Error writing to file: " << strerror(errno) << endl);
      }
      return;
   }
   memcpy(((char *)m_cache) + m_usedcache, data, num_bytes);
   m_usedcache += num_bytes;
}
